from cached_property import cached_property
from csv import writer
from decimal import Decimal
from decimal import InvalidOperation
from onegov.core.collection import Pagination
from onegov.swissvotes.fields.dataset import COLUMNS
from onegov.swissvotes.models import PolicyArea
from onegov.swissvotes.models import SwissVote
from sqlalchemy import func
from sqlalchemy import or_
from xlsxwriter.workbook import Workbook


class SwissVoteCollection(Pagination):

    batch_size = 20
    initial_sort_by = 'date'
    initial_sort_order = 'descending'
    default_sort_order = 'ascending'

    SORT_BYS = (
        'date',
        'legal_form',
        'result',
        'result_people_yeas_p',
        'title'
    )
    SORT_ORDERS = ('ascending', 'descending')

    def __init__(
        self,
        session,
        page=None,
        from_date=None,
        to_date=None,
        legal_form=None,
        result=None,
        policy_area=None,
        term=None,
        full_text=None,
        position_federal_council=None,
        position_national_council=None,
        position_council_of_states=None,
        sort_by=None,
        sort_order=None
    ):
        self.session = session
        self.page = page
        self.from_date = from_date
        self.to_date = to_date
        self.legal_form = legal_form
        self.result = result
        self.policy_area = policy_area
        self.term = term
        self.full_text = full_text
        self.position_federal_council = position_federal_council
        self.position_national_council = position_national_council
        self.position_council_of_states = position_council_of_states
        self.sort_by = sort_by
        self.sort_order = sort_order

    def add(self, **kwargs):
        vote = SwissVote(**kwargs)
        self.session.add(vote)
        self.session.flush()
        return vote

    def subset(self):
        return self.query()

    def __eq__(self, other):
        return (
            (self.page or 0) == (other.page or 0) and
            (self.from_date or None) == (other.from_date or None) and
            (self.to_date or None) == (other.to_date or None) and
            set(self.legal_form or []) == set(other.legal_form or []) and
            set(self.result or []) == set(other.result or []) and
            set(self.policy_area or []) == set(other.policy_area or []) and
            (self.term or None) == (other.term or None) and
            (self.full_text or None) == (other.full_text or None) and
            (
                (self.position_federal_council or []) ==
                (other.position_federal_council or [])
            ) and
            (
                (self.position_national_council or []) ==
                (other.position_national_council or [])
            ) and
            (
                (self.position_council_of_states or []) ==
                (other.position_council_of_states or [])
            ) and
            (self.sort_by or None) == (other.sort_by or None) and
            (self.sort_order or None) == (other.sort_order or None)
        )

    def default(self):
        """ Returns the votes unfiltered and ordered by default. """

        return self.__class__(self.session)

    @property
    def page_index(self):
        """ The current page. """

        return self.page or 0

    def page_by_index(self, page):
        """ Returns the requested page. """

        return self.__class__(
            self.session,
            page=page,
            from_date=self.from_date,
            to_date=self.to_date,
            legal_form=self.legal_form,
            result=self.result,
            policy_area=self.policy_area,
            term=self.term,
            full_text=self.full_text,
            position_federal_council=self.position_federal_council,
            position_national_council=self.position_national_council,
            position_council_of_states=self.position_council_of_states,
            sort_by=self.sort_by,
            sort_order=self.sort_order
        )

    @property
    def current_sort_by(self):
        """ Returns the currently used sorting key.

        Defaults to a reasonable value.

        """
        result = self.initial_sort_by if self.sort_by is None else self.sort_by
        return result if result in self.SORT_BYS else self.initial_sort_by

    @property
    def current_sort_order(self):
        """ Returns the currently used sorting order.

        Defaults to a reasonable value.

        """

        default = self.default_sort_order
        default = self.initial_sort_order if self.sort_by is None else default
        result = default if self.sort_order is None else self.sort_order
        return result if result in self.SORT_ORDERS else default

    def sort_order_by_key(self, sort_by):
        """ Returns the sort order by key.

        Defaults to 'unsorted'.

        """

        if self.current_sort_by == sort_by:
            return self.current_sort_order
        return 'unsorted'

    def by_order(self, sort_by):
        """ Returns the votes ordered by the given key. """

        sort_order = self.default_sort_order
        if sort_by == self.current_sort_by:
            if self.current_sort_order == 'ascending':
                sort_order = 'descending'
            else:
                sort_order = 'ascending'

        return self.__class__(
            self.session,
            page=None,
            from_date=self.from_date,
            to_date=self.to_date,
            legal_form=self.legal_form,
            result=self.result,
            policy_area=self.policy_area,
            term=self.term,
            full_text=self.full_text,
            position_federal_council=self.position_federal_council,
            position_national_council=self.position_national_council,
            position_council_of_states=self.position_council_of_states,
            sort_by=sort_by,
            sort_order=sort_order
        )

    @property
    def order_by(self):
        """ Returns an SqlAlchemy expression for ordering queries based
        on the current sorting key and ordering.

        """

        if self.current_sort_by == 'title':
            from onegov.core.orm.func import unaccent
            result = unaccent(SwissVote.title)
        else:
            result = (
                getattr(SwissVote, f'_{self.current_sort_by}', None) or
                getattr(SwissVote, self.current_sort_by, None)
            )
            if not result:
                raise NotImplementedError()

        if self.current_sort_order == 'descending':
            result = result.desc()

        return result

    @property
    def offset(self):
        """ The current position in the batch. """

        return (self.page or 0) * self.batch_size

    @property
    def previous(self):
        """ The previous page. """

        if (self.page or 0) - 1 >= 0:
            return self.page_by_index((self.page or 0) - 1)

    @property
    def next(self):
        """ The next page. """

        if (self.page or 0) + 1 < self.pages_count:
            return self.page_by_index((self.page or 0) + 1)

    @property
    def term_filter_numeric(self):
        """ Returns a list of SqlAlchemy filter statements matching possible
        numeric attributes based on the term.

        """

        result = []
        if self.term:
            for part in self.term.split():
                if part.replace('.', '').isnumeric():
                    number = Decimal(part)
                    result.append(SwissVote.bfs_number == number)
                    result.append(SwissVote.procedure_number == number)

        return result

    @property
    def term_filter_text(self):
        """ Returns a list of SqlAlchemy filter statements matching possible
        fulltext attributes based on the term.

        """

        if not self.term:
            return []

        # Once we upgrade to PostgreSql 11, we can use websearch_to_tsquery
        term = ''.join((c for c in self.term if c.isalnum() or c in ',. '))
        term = ' <-> '.join([part for part in term.split()])

        def match_convert(column, language='german'):
            return func.to_tsvector(language, column).\
                match(term, postgresql_regconfig='german')

        def match(column, language='german'):
            return column.match(term, postgresql_regconfig='german')

        if not self.full_text:
            return [
                match_convert(SwissVote.title),
                match_convert(SwissVote.keyword),
            ]
        return [
            match_convert(SwissVote.title),
            match_convert(SwissVote.keyword),
            match_convert(SwissVote.initiator),
            match(SwissVote.searchable_text_de_CH),
            match(SwissVote.searchable_text_fr_CH, 'french'),
        ]

    @property
    def term_filter(self):
        """ Returns a list of SqlAlchemy filter statements based on the search
        term.

        """

        return self.term_filter_numeric + self.term_filter_text

    def query(self):
        """ Returns the votes matching to the current filters and order. """

        query = self.session.query(SwissVote)

        def in_or_none(column, values):
            statement = column.in_(values)
            if -1 in values:
                statement = or_(statement, column.is_(None))
            return statement

        if self.from_date:
            query = query.filter(SwissVote.date >= self.from_date)
        if self.to_date:
            query = query.filter(SwissVote.date <= self.to_date)
        if self.legal_form:
            query = query.filter(SwissVote._legal_form.in_(self.legal_form))
        if self.result:
            query = query.filter(SwissVote._result.in_(self.result))
        if self.policy_area:
            levels = [[], [], []]
            for area in self.policy_area:
                area = PolicyArea(area)
                if area.level == 1:
                    levels[0].append(area.descriptor_decimal)
                if area.level == 2:
                    levels[1].append(area.descriptor_decimal)
                if area.level == 3:
                    levels[2].append(area.descriptor_decimal)
            if levels[0]:
                query = query.filter(
                    or_(
                        SwissVote.descriptor_1_level_1.in_(levels[0]),
                        SwissVote.descriptor_2_level_1.in_(levels[0]),
                        SwissVote.descriptor_3_level_1.in_(levels[0])
                    )
                )
            if levels[1]:
                query = query.filter(
                    or_(
                        SwissVote.descriptor_1_level_2.in_(levels[1]),
                        SwissVote.descriptor_2_level_2.in_(levels[1]),
                        SwissVote.descriptor_3_level_2.in_(levels[1])
                    )
                )
            if levels[2]:
                query = query.filter(
                    or_(
                        SwissVote.descriptor_1_level_3.in_(levels[2]),
                        SwissVote.descriptor_2_level_3.in_(levels[2]),
                        SwissVote.descriptor_3_level_3.in_(levels[2])
                    )
                )
        if self.term:
            query = query.filter(or_(*self.term_filter))
        if self.position_federal_council:
            query = query.filter(
                in_or_none(
                    SwissVote._position_federal_council,
                    self.position_federal_council
                )
            )
        if self.position_national_council:
            query = query.filter(
                in_or_none(
                    SwissVote._position_national_council,
                    self.position_national_council
                )
            )
        if self.position_council_of_states:
            query = query.filter(
                in_or_none(
                    SwissVote._position_council_of_states,
                    self.position_council_of_states
                )
            )

        query = query.order_by(self.order_by)

        return query

    def by_bfs_number(self, bfs_number):
        """ Returns the vote with the given BFS number. """
        try:
            bfs_number = Decimal(bfs_number)
        except InvalidOperation:
            return None

        query = self.query().filter(SwissVote.bfs_number == bfs_number)
        return query.first()

    @cached_property
    def available_descriptors(self):
        """ Returns a list of the used descriptor values (level 1-3). """

        query = self.session.query
        return [
            set([
                x[0] for x in query(SwissVote.descriptor_1_level_1).union(
                    query(SwissVote.descriptor_2_level_1),
                    query(SwissVote.descriptor_3_level_1)
                ).all() if x[0]
            ]),
            set([
                x[0] for x in query(SwissVote.descriptor_1_level_2).union(
                    query(SwissVote.descriptor_2_level_2),
                    query(SwissVote.descriptor_3_level_2)
                ).all() if x[0]
            ]),
            set([
                x[0] for x in query(SwissVote.descriptor_1_level_3).union(
                    query(SwissVote.descriptor_2_level_3),
                    query(SwissVote.descriptor_3_level_3)
                ).all() if x[0]
            ]),
        ]

    def update(self, votes):
        """ Adds or updates the given votes. """

        added = 0
        updated = 0
        query = self.session.query(SwissVote)
        for vote in votes:
            old = query.filter_by(bfs_number=vote.bfs_number).first()
            if old:
                changed = False
                for attribute in COLUMNS.keys():
                    value = getattr(vote, attribute)
                    if getattr(old, attribute) != value:
                        setattr(old, attribute, value)
                        changed = True

                if changed:
                    updated += 1
            else:
                added += 1
                self.session.add(vote)

        return added, updated

    def export_csv(self, file):
        """ Exports all votes according to the code book. """

        csv = writer(file)
        csv.writerow(COLUMNS.values())

        for vote in self.query().order_by(None).order_by(SwissVote.bfs_number):
            row = []
            for attribute in COLUMNS.keys():
                value = getattr(vote, attribute)
                type_ = str(vote.__table__.columns[attribute.lstrip('_')].type)
                if value is None:
                    row.append('.')
                elif type_ == 'TEXT':
                    row.append(value)
                elif type_ == 'DATE':
                    row.append(f'{value:%d.%m.%Y}')
                elif type_ == 'INTEGER':
                    row.append(str(value))
                elif type_ == 'INT4RANGE':
                    row.append(f'{value.lower}-{value.upper}')
                elif type_.startswith('NUMERIC'):
                    row.append(
                        f'{value:f}'.replace('.', ',').rstrip('0').rstrip(',')
                    )
            csv.writerow(row)

    def export_xlsx(self, file):
        """ Exports all votes according to the code book. """

        workbook = Workbook(file, {'default_date_format': 'dd.mm.yyyy'})
        worksheet = workbook.add_worksheet()
        worksheet.write_row(0, 0, COLUMNS.values())

        row = 0
        for vote in self.query().order_by(None).order_by(SwissVote.bfs_number):
            row += 1
            for column_, attribute in enumerate(COLUMNS.keys()):
                value = getattr(vote, attribute)
                type_ = str(vote.__table__.columns[attribute.lstrip('_')].type)
                if value is None:
                    pass
                elif type_ == 'TEXT':
                    worksheet.write_string(row, column_, value)
                elif type_ == 'DATE':
                    worksheet.write_datetime(row, column_, value)
                elif type_ == 'INTEGER' or type_.startswith('NUMERIC'):
                    worksheet.write_number(row, column_, value)
                elif type_ == 'INT4RANGE':
                    worksheet.write_string(
                        row, column_, f'{value.lower}-{value.upper}'
                    )

        workbook.close()



from cached_property import cached_property
from onegov.core import Framework
from onegov.core import utils
from onegov.file import DepotApp
from onegov.form import FormApp
from onegov.quill import QuillApp
from onegov.swissvotes import _
from onegov.swissvotes.models import Principal
from onegov.swissvotes.models import TranslatablePage
from onegov.swissvotes.theme import SwissvotesTheme


class SwissvotesApp(Framework, FormApp, QuillApp, DepotApp):
    """ The swissvotes application. Include this in your onegov.yml to serve
    it with onegov-server.

    """

    serve_static_files = True

    def configure_sentry(self, **cfg):
        self.sentry_js = cfg.get('sentry_js')

    @cached_property
    def principal(self):
        return Principal()

    def add_initial_content(self):
        translators = {
            locale: self.translations.get(locale) for locale in self.locales
        }
        session = self.session()
        for page, title in (
            ('home', _("Homepage")),
            ('dataset', _("Dataset")),
            ('about', _("About")),
            ('contact', _("Contact")),
            ('disclaimer', _("Disclaimer")),
            ('imprint', _("Imprint")),
        ):
            translations = {
                locale: title.interpolate(translators[locale].gettext(title))
                for locale in self.locales
            }
            session.add(
                TranslatablePage(
                    id=page,
                    title_translations=translations,
                    content_translations=translations
                )
            )


@SwissvotesApp.static_directory()
def get_static_directory():
    return 'static'


@SwissvotesApp.template_directory()
def get_template_directory():
    return 'templates'


@SwissvotesApp.setting(section='core', name='theme')
def get_theme():
    return SwissvotesTheme()


@SwissvotesApp.setting(section='i18n', name='localedirs')
def get_i18n_localedirs():
    return [
        utils.module_path('onegov.swissvotes', 'locale'),
        utils.module_path('onegov.form', 'locale'),
        utils.module_path('onegov.user', 'locale')
    ]


@SwissvotesApp.setting(section='i18n', name='locales')
def get_i18n_used_locales():
    return {'de_CH', 'fr_CH', 'en_US'}


@SwissvotesApp.setting(section='i18n', name='default_locale')
def get_i18n_default_locale():
    return 'de_CH'


@SwissvotesApp.webasset_path()
def get_shared_assets_path():
    return utils.module_path('onegov.shared', 'assets/js')


@SwissvotesApp.webasset_path()
def get_js_path():
    return 'assets/js'


@SwissvotesApp.webasset_path()
def get_css_path():
    return 'assets/css'


@SwissvotesApp.webasset_output()
def get_webasset_output():
    return 'assets/bundles'


@SwissvotesApp.webasset('frameworks')
def get_frameworks_asset():
    yield 'modernizr.js'
    yield 'jquery.js'
    yield 'jquery.tablesorter.js'
    yield 'tablesaw.css'
    yield 'tablesaw.jquery.js'
    yield 'tablesaw-create.js'
    yield 'tablesaw-init.js'
    yield 'foundation.js'
    yield 'underscore.js'
    yield 'react.js'
    yield 'react-dom.js'
    yield 'react-dropdown-tree-select.js'
    yield 'react-dropdown-tree-select.css'
    yield 'form_dependencies.js'


@SwissvotesApp.webasset('common')
def get_common_asset():
    yield 'common.js'
    yield 'policy-selector.jsx'



from webtest import TestApp as Client


def test_view_page(swissvotes_app):
    client = Client(swissvotes_app)

    for locale, text in (
        ('de_CH', "Über uns"),
        ('fr_CH', "À propos de nous"),
        ('en_US', "About")
    ):
        client.get(f'/locale/{locale}').follow()
        assert text in client.get('/page/about')

    login = client.get('/auth/login')
    login.form['username'] = 'admin@example.org'
    login.form['password'] = 'hunter2'
    login.form.submit()

    for locale, title, content in (
        ('de_CH', "Über das Projekt", "Platzhalterttext"),
        ('fr_CH', "À propos du projet", "Texte de remplacement"),
        ('en_US', "About the project", "Placeholder text")
    ):
        client.get(f'/locale/{locale}').follow()
        page = client.get('/page/about/edit')
        page.form['title'] = title
        page.form['content'] = content
        page.form.submit()

    for locale, title, content in (
        ('de_CH', "Über das Projekt", "Platzhalterttext"),
        ('fr_CH', "À propos du projet", "Texte de remplacement"),
        ('en_US', "About the project", "Placeholder text")
    ):
        client.get(f'/locale/{locale}').follow()
        page = client.get('/page/about')
        assert title in page
        assert content in page



from decimal import Decimal
from onegov.swissvotes import _
from onegov.swissvotes.collections import SwissVoteCollection
from onegov.swissvotes.layouts import DefaultLayout
from onegov.swissvotes.layouts import DeleteVoteLayout
from onegov.swissvotes.layouts import DeleteVotesLayout
from onegov.swissvotes.layouts import EditPageLayout
from onegov.swissvotes.layouts import MailLayout
from onegov.swissvotes.layouts import PageLayout
from onegov.swissvotes.layouts import UpdateVotesLayout
from onegov.swissvotes.layouts import UploadVoteAttachemtsLayout
from onegov.swissvotes.layouts import VoteLayout
from onegov.swissvotes.layouts import VotesLayout
from onegov.swissvotes.layouts import VoteStrengthsLayout
from onegov.swissvotes.models import SwissVote
from onegov.swissvotes.models import TranslatablePage


class DummyPrincipal(object):
    pass


class DummyApp(object):
    principal = DummyPrincipal()
    theme_options = {}


class DummyRequest(object):
    app = DummyApp()
    is_logged_in = False
    locale = 'de_CH'
    roles = []
    includes = []
    session = None
    url = ''

    def has_role(self, *roles):
        return any((role in self.roles for role in roles))

    def translate(self, text):
        return str(text)

    def include(self, *args, **kwargs):
        self.includes.extend(args)

    def link(self, model, name=''):
        if isinstance(model, str):
            return f'{model}/{name}'
        return f'{model.__class__.__name__}/{name}'

    def exclude_invisible(self, objects):
        return objects

    def new_csrf_token(self):
        return 'x'


def test_layout_default(swissvotes_app):
    request = DummyRequest()
    request.app = swissvotes_app
    request.session = swissvotes_app.session()
    model = None

    layout = DefaultLayout(model, request)
    assert layout.title == ""
    assert layout.editbar_links == []
    assert layout.breadcrumbs == [('Homepage', 'Principal/', 'current')]
    assert layout.static_path == 'Principal/static'
    assert layout.app_version
    assert layout.locales == [
        ('de_CH', 'de', 'Deutsch', 'SiteLocale/'),
        ('fr_CH', 'fr', 'Français', 'SiteLocale/'),
        ('en_US', 'en', 'English', 'SiteLocale/')
    ]
    assert layout.request.includes == ['frameworks', 'chosen', 'common']
    assert layout.top_navigation == [
        ('Votes', 'SwissVoteCollection/'),
        ('Datensatz', 'TranslatablePage/'),
        ('Über uns', 'TranslatablePage/'),
        ('Kontakt', 'TranslatablePage/')
    ]
    assert layout.homepage_link == 'Principal/'
    assert layout.disclaimer_link == ('Disclaimer', 'TranslatablePage/')
    assert layout.imprint_link == ('Impressum', 'TranslatablePage/')
    assert layout.votes_link == 'SwissVoteCollection/'
    assert layout.login_link == 'Auth/login'
    assert layout.logout_link is None

    assert layout.format_bfs_number(Decimal('100')) == '100'
    assert layout.format_bfs_number(Decimal('100.1')) == '100.1'
    assert layout.format_bfs_number(Decimal('100.12')) == '100.1'

    # Login
    request.is_logged_in = True
    layout = DefaultLayout(model, request)
    assert layout.login_link is None
    assert layout.logout_link == 'Auth/logout'

    # Remove initial content
    layout.pages.query().delete()
    assert layout.top_navigation == [('Votes', 'SwissVoteCollection/')]
    assert layout.disclaimer_link == ('', '')
    assert layout.imprint_link == ('', '')

    assert layout.format_policy_areas(SwissVote()) == ''

    vote = SwissVote(
        descriptor_1_level_1=Decimal('4'),
        descriptor_2_level_1=Decimal('8'),
        descriptor_2_level_2=Decimal('8.3'),
        descriptor_3_level_1=Decimal('10'),
        descriptor_3_level_2=Decimal('10.3'),
        descriptor_3_level_3=Decimal('10.33'),
    )
    assert layout.format_policy_areas(vote) == (
        '<span title="d-1-10 &gt; d-2-103 &gt; d-3-1033">d-1-10</span>,<br>'
        '<span title="d-1-4">d-1-4</span>,<br>'
        '<span title="d-1-8 &gt; d-2-83">d-1-8</span>'
    )

    vote = SwissVote(
        descriptor_2_level_1=Decimal('10'),
        descriptor_2_level_2=Decimal('10.3'),
        descriptor_3_level_1=Decimal('10'),
        descriptor_3_level_2=Decimal('10.3'),
        descriptor_3_level_3=Decimal('10.33'),
    )
    assert layout.format_policy_areas(vote) == (
        '<span title="d-1-10 &gt; d-2-103 &#10;&#10;'
        'd-1-10 &gt; d-2-103 &gt; d-3-1033">d-1-10</span>'
    )


def test_layout_mail():
    request = DummyRequest()
    model = None

    layout = MailLayout(model, request)
    assert layout.primary_color == '#fff'


def test_layout_page(session):
    request = DummyRequest()
    model = TranslatablePage(
        id='page',
        title_translations={'en_US': "Page", 'de_CH': "Seite"},
        content_translations={'en_US': "Content", 'de_CH': "Inhalt"}
    )
    session.add(model)
    session.flush()

    layout = PageLayout(model, request)
    assert layout.title == "Seite"
    assert layout.editbar_links == []
    assert layout.breadcrumbs == [
        ('Homepage', 'DummyPrincipal/', ''),
        ('Seite', '#', 'current')
    ]

    # Log in as editor
    request.roles = ['editor']
    layout = PageLayout(model, request)
    assert layout.editbar_links == [
        ('Edit page', 'TranslatablePage/edit', 'edit-icon')
    ]

    # Log in as admin
    request.roles = ['admin']
    layout = PageLayout(model, request)
    assert layout.editbar_links == [
        ('Edit page', 'TranslatablePage/edit', 'edit-icon')
    ]


def test_layout_edit_page(session):
    request = DummyRequest()
    model = TranslatablePage(
        id='page',
        title_translations={'en_US': "Page", 'de_CH': "Seite"},
        content_translations={'en_US': "Content", 'de_CH': "Inhalt"}
    )
    session.add(model)
    session.flush()

    layout = EditPageLayout(model, request)
    assert layout.title == "Edit page"
    assert layout.editbar_links == []
    assert layout.breadcrumbs == [
        ('Homepage', 'DummyPrincipal/', ''),
        ('Seite', 'TranslatablePage/', ''),
        ('Edit page', '#', 'current')
    ]

    # Log in as editor
    request.roles = ['editor']
    layout = EditPageLayout(model, request)
    assert layout.editbar_links == []

    # Log in as admin
    request.roles = ['admin']
    layout = EditPageLayout(model, request)
    assert layout.editbar_links == []


def test_layout_vote():
    request = DummyRequest()
    model = SwissVote(title="Vote")

    layout = VoteLayout(model, request)
    assert layout.title == "Vote"
    assert layout.editbar_links == []
    assert layout.breadcrumbs == [
        ('Homepage', 'DummyPrincipal/', ''),
        ('Votes', 'SwissVoteCollection/', ''),
        ('Vote', '#', 'current')
    ]

    # Log in as editor
    request.roles = ['editor']
    layout = VoteLayout(model, request)
    assert layout.editbar_links == [
        ('Manage attachments', 'SwissVote/upload', 'upload-icon'),
        ('Delete vote', 'SwissVote/delete', 'delete-icon')
    ]

    # Log in as admin
    request.roles = ['admin']
    layout = VoteLayout(model, request)
    assert layout.editbar_links == [
        ('Manage attachments', 'SwissVote/upload', 'upload-icon'),
        ('Delete vote', 'SwissVote/delete', 'delete-icon')
    ]


def test_layout_vote_strengths():
    request = DummyRequest()
    model = SwissVote(title="Vote")

    layout = VoteStrengthsLayout(model, request)
    assert layout.title == _("Voter strengths")
    assert layout.editbar_links == []
    assert layout.breadcrumbs == [
        ('Homepage', 'DummyPrincipal/', ''),
        ('Votes', 'SwissVoteCollection/', ''),
        ('Vote', 'SwissVote/', ''),
        ('Voter strengths', '#', 'current'),
    ]

    # Log in as editor
    request.roles = ['editor']
    layout = VoteStrengthsLayout(model, request)
    assert layout.editbar_links == []

    # Log in as admin
    request.roles = ['admin']
    layout = VoteStrengthsLayout(model, request)
    assert layout.editbar_links == []


def test_layout_upload_vote_attachemts():
    request = DummyRequest()
    model = SwissVote(title="Vote")

    layout = UploadVoteAttachemtsLayout(model, request)
    assert layout.title == _("Manage attachments")
    assert layout.editbar_links == []
    assert layout.breadcrumbs == [
        ('Homepage', 'DummyPrincipal/', ''),
        ('Votes', 'SwissVoteCollection/', ''),
        ('Vote', 'SwissVote/', ''),
        ('Manage attachments', '#', 'current'),
    ]

    # Log in as editor
    request.roles = ['editor']
    layout = UploadVoteAttachemtsLayout(model, request)
    assert layout.editbar_links == []

    # Log in as admin
    request.roles = ['admin']
    layout = UploadVoteAttachemtsLayout(model, request)
    assert layout.editbar_links == []


def test_layout_delete_vote():
    request = DummyRequest()
    model = SwissVote(title="Vote")

    layout = DeleteVoteLayout(model, request)
    assert layout.title == _("Delete vote")
    assert layout.editbar_links == []
    assert layout.breadcrumbs == [
        ('Homepage', 'DummyPrincipal/', ''),
        ('Votes', 'SwissVoteCollection/', ''),
        ('Vote', 'SwissVote/', ''),
        ('Delete vote', '#', 'current'),
    ]

    # Log in as editor
    request.roles = ['editor']
    layout = DeleteVoteLayout(model, request)
    assert layout.editbar_links == []

    # Log in as admin
    request.roles = ['admin']
    layout = DeleteVoteLayout(model, request)
    assert layout.editbar_links == []


def test_layout_votes():
    request = DummyRequest()
    model = SwissVoteCollection(None)

    layout = VotesLayout(model, request)
    assert layout.title == _("Votes")
    assert layout.editbar_links == []
    assert layout.breadcrumbs == [
        ('Homepage', 'DummyPrincipal/', ''),
        ('Votes', 'SwissVoteCollection/', 'current')
    ]

    # Log in as editor
    request.roles = ['editor']
    layout = VotesLayout(model, request)
    assert layout.editbar_links == [
        ('Update dataset', 'SwissVoteCollection/update', 'upload-icon'),
        ('Download dataset (CSV)', 'SwissVoteCollection/csv', 'export-icon'),
        ('Download dataset (XLSX)', 'SwissVoteCollection/xlsx', 'export-icon')
    ]

    # Log in as admin
    request.roles = ['admin']
    layout = VotesLayout(model, request)
    assert layout.editbar_links == [
        ('Update dataset', 'SwissVoteCollection/update', 'upload-icon'),
        ('Download dataset (CSV)', 'SwissVoteCollection/csv', 'export-icon'),
        ('Download dataset (XLSX)', 'SwissVoteCollection/xlsx', 'export-icon'),
        ('Delete all votes', 'SwissVoteCollection/delete', 'delete-icon')
    ]


def test_layout_update_votes():
    request = DummyRequest()
    model = SwissVoteCollection(None)

    layout = UpdateVotesLayout(model, request)
    assert layout.title == _("Update dataset")
    assert layout.editbar_links == []
    assert layout.breadcrumbs == [
        ('Homepage', 'DummyPrincipal/', ''),
        ('Votes', 'SwissVoteCollection/', ''),
        ('Update dataset', '#', 'current')
    ]

    # Log in as editor
    request.roles = ['editor']
    layout = UpdateVotesLayout(model, request)
    assert layout.editbar_links == []

    # Log in as admin
    request.roles = ['admin']
    layout = UpdateVotesLayout(model, request)
    assert layout.editbar_links == []


def test_layout_delete_votes():
    request = DummyRequest()
    model = SwissVoteCollection(None)

    layout = DeleteVotesLayout(model, request)
    assert layout.title == _("Delete all votes")
    assert layout.editbar_links == []
    assert layout.breadcrumbs == [
        ('Homepage', 'DummyPrincipal/', ''),
        ('Votes', 'SwissVoteCollection/', ''),
        ('Delete all votes', '#', 'current')
    ]

    # Log in as editor
    request.roles = ['editor']
    layout = DeleteVotesLayout(model, request)
    assert layout.editbar_links == []

    # Log in as admin
    request.roles = ['admin']
    layout = DeleteVotesLayout(model, request)
    assert layout.editbar_links == []



from cached_property import cached_property
from onegov.swissvotes import _
from onegov.swissvotes.layouts.default import DefaultLayout


class VoteLayout(DefaultLayout):

    @cached_property
    def title(self):
        return self.model.title

    @cached_property
    def editbar_links(self):
        if not self.request.has_role('admin', 'editor'):
            return []
        return [
            (
                _("Manage attachments"),
                self.request.link(self.model, name='upload'),
                'upload-icon'
            ),
            (
                _("Delete vote"),
                self.request.link(self.model, name='delete'),
                'delete-icon'
            )
        ]

    @cached_property
    def breadcrumbs(self):
        return [
            (_("Homepage"), self.homepage_link, ''),
            (_("Votes"), self.votes_link, ''),
            (self.title, '#', 'current'),
        ]


class VoteStrengthsLayout(DefaultLayout):

    @cached_property
    def title(self):
        return _("Voter strengths")

    @cached_property
    def breadcrumbs(self):
        return [
            (_("Homepage"), self.homepage_link, ''),
            (_("Votes"), self.votes_link, ''),
            (self.model.title, self.request.link(self.model), ''),
            (self.title, '#', 'current'),
        ]


class UploadVoteAttachemtsLayout(DefaultLayout):

    @cached_property
    def title(self):
        return _("Manage attachments")

    @cached_property
    def breadcrumbs(self):
        return [
            (_("Homepage"), self.homepage_link, ''),
            (_("Votes"), self.votes_link, ''),
            (self.model.title, self.request.link(self.model), ''),
            (self.title, '#', 'current'),
        ]


class DeleteVoteLayout(DefaultLayout):

    @cached_property
    def title(self):
        return _("Delete vote")

    @cached_property
    def breadcrumbs(self):
        return [
            (_("Homepage"), self.homepage_link, ''),
            (_("Votes"), self.votes_link, ''),
            (self.model.title, self.request.link(self.model), ''),
            (self.title, '#', 'current'),
        ]



from cached_property import cached_property
from onegov.swissvotes import _
from onegov.swissvotes.layouts.default import DefaultLayout


class PageLayout(DefaultLayout):

    @cached_property
    def title(self):
        return self.model.title

    @cached_property
    def editbar_links(self):
        if not self.request.has_role('admin', 'editor'):
            return []
        return [
            (
                _("Edit page"),
                self.request.link(self.model, name='edit'),
                'edit-icon'
            )
        ]

    @cached_property
    def breadcrumbs(self):
        if self.model.id == 'home':
            return [(_("Homepage"), self.homepage_link, 'current')]

        return [
            (_("Homepage"), self.homepage_link, ''),
            (self.title, '#', 'current'),
        ]


class EditPageLayout(DefaultLayout):

    @cached_property
    def title(self):
        return _("Edit page")

    @cached_property
    def breadcrumbs(self):
        return [
            (_("Homepage"), self.homepage_link, ''),
            (self.model.title, self.request.link(self.model), ''),
            (self.title, '#', 'current'),
        ]



from onegov.swissvotes.collections import TranslatablePageCollection


def test_app_principal(swissvotes_app):
    assert swissvotes_app.principal


def test_app_initial_content(swissvotes_app):
    pages = TranslatablePageCollection(swissvotes_app.session())

    assert {page.title for page in pages.query()} == {
        'Startseite',
        'Datensatz',
        'Über uns',
        'Kontakt',
        'Disclaimer',
        'Impressum'
    }



from onegov.core.utils import module_path
from pytest import mark
from webtest import TestApp as Client
from webtest.forms import Upload
from onegov.swissvotes.models import SwissVote


@mark.parametrize("file", [
    module_path('onegov.swissvotes', 'tests/fixtures/votes.xlsx'),
])
def test_update_votes(swissvotes_app, file):
    client = Client(swissvotes_app)
    client.get('/locale/de_CH').follow()

    login = client.get('/auth/login')
    login.form['username'] = 'admin@example.org'
    login.form['password'] = 'hunter2'
    login.form.submit()

    with open(file, 'rb') as f:
        content = f.read()

    # Upload
    manage = client.get('/votes/update')
    manage.form['dataset'] = Upload(
        'votes.xlsx',
        content,
        'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
    )
    manage = manage.form.submit().follow()
    assert "Datensatz aktualisiert (634 hinzugefügt, 0 geändert)" in manage

    session = swissvotes_app.session()
    vote = session.query(SwissVote).filter_by(bfs_number=82.2).one()
    assert str(vote.bfs_number) == '82.20'
    assert vote.date.isoformat() == '1920-03-21'
    assert vote.legislation_number == 25
    assert vote.decade.lower == 1920
    assert vote.decade.upper == 1929
    assert vote.title == (
        "Gegenentwurf zur Volksinitiative "
        "«für ein Verbot der Errichtung von Spielbanken»"
    )
    assert [str(pa) for pa in vote.policy_areas] == ['4.41.413', '4.44.443']
    assert str(vote.result_turnout) == '60.2323410000'
    assert vote.recommendations_parties['Nay'][0].name == 'sps'

    # Upload (unchanged)
    manage = client.get('/votes/update')
    manage.form['dataset'] = Upload(
        'votes.xlsx',
        content,
        'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
    )
    manage = manage.form.submit().follow()
    assert "Datensatz aktualisiert (0 hinzugefügt, 0 geändert)" in manage

    # Download
    csv = client.get('/votes/csv').body
    xlsx = client.get('/votes/xlsx').body

    # Upload (roundtrip)
    manage = client.get('/votes/update')
    manage.form['dataset'] = Upload(
        'votes.xlsx',
        xlsx,
        'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
    )
    manage = manage.form.submit().follow()
    assert "Datensatz aktualisiert (0 hinzugefügt, 0 geändert)" in manage

    assert csv == client.get('/votes/csv').body



from datetime import date
from decimal import Decimal
from onegov.swissvotes.models.actor import Actor
from onegov.swissvotes.models.localized_file import LocalizedFile
from onegov.swissvotes.models.page import TranslatablePage
from onegov.swissvotes.models.policy_area import PolicyArea
from onegov.swissvotes.models.principal import Principal
from onegov.swissvotes.models.vote import SwissVote
from psycopg2.extras import NumericRange
from translationstring import TranslationString


class DummyRequest(object):
    def translate(self, text):
        if isinstance(text, TranslationString):
            return text.interpolate()
        return text


def test_actor():
    actor = Actor('csp')
    assert actor.name == 'csp'
    assert actor.abbreviation == 'actor-csp-abbreviation'
    assert isinstance(actor.abbreviation, TranslationString)
    assert actor.label == 'actor-csp-label'
    assert isinstance(actor.label, TranslationString)
    assert actor.html(DummyRequest()) == (
        '<span title="actor-csp-label">actor-csp-abbreviation</span>'
    )

    actor = Actor('xxx')
    assert actor.name == 'xxx'
    assert actor.abbreviation == 'XXX'
    assert not isinstance(actor.abbreviation, TranslationString)
    assert actor.label == 'XXX'
    assert not isinstance(actor.label, TranslationString)
    assert actor.html(DummyRequest()) == '<span title="XXX">XXX</span>'

    assert Actor('csp') == Actor('csp')
    assert Actor('csp') != Actor('xxx')


def test_localized_file():
    class SessionManager(object):
        def __init__(self):
            self.current_locale = 'de_CH'

    class MyClass(object):
        file = LocalizedFile()

        def __init__(self):
            self.session_manager = SessionManager()
            self.files = []

    class File(object):
        def __init__(self, name):
            self.name = name

    my = MyClass()
    assert my.file is None

    # Add CH
    my.file = File('A')
    assert my.file.name == 'file-de_CH'
    assert set(file.name for file in my.files) == {'file-de_CH'}

    # Add FR
    my.session_manager.current_locale = 'fr_CH'
    assert my.file is None

    my.file = File('B')
    assert my.file.name == 'file-fr_CH'
    assert set(file.name for file in my.files) == {'file-de_CH', 'file-fr_CH'}

    # Access unrestricted
    assert MyClass.__dict__['file'].__get_by_locale__(my, 'de_CH').name == \
        'file-de_CH'
    assert MyClass.__dict__['file'].__get_by_locale__(my, 'fr_CH').name == \
        'file-fr_CH'
    assert MyClass.__dict__['file'].__get_by_locale__(my, 'rm_CH') is None

    # Delete FR
    del my.file
    assert my.file is None
    assert set(file.name for file in my.files) == {'file-de_CH'}


def test_page(session):
    session.add(
        TranslatablePage(
            id='page',
            title_translations={'de_CH': "Titel", 'en': "Title"},
            content_translations={'de_CH': "Inhalt", 'en': "Content"}
        )
    )
    session.flush()

    page = session.query(TranslatablePage).one()
    assert page.id == 'page'
    assert page.title_translations == {'de_CH': "Titel", 'en': "Title"}
    assert page.content_translations == {'de_CH': "Inhalt", 'en': "Content"}


def test_policy_area(session):
    policy_area = PolicyArea('1')
    assert repr(policy_area) == '1'
    assert policy_area.level == 1
    assert policy_area.descriptor == 1
    assert policy_area.descriptor_path == [1]
    assert policy_area.descriptor_decimal == Decimal('1')
    assert policy_area.label == 'd-1-1'
    assert policy_area.label_path == ['d-1-1']
    assert policy_area.html(DummyRequest()) == '<span>d-1-1</span>'

    policy_area = PolicyArea('1.12')
    assert repr(policy_area) == '1.12'
    assert policy_area.level == 2
    assert policy_area.descriptor == 12
    assert policy_area.descriptor_path == [1, 12]
    assert policy_area.descriptor_decimal == Decimal('1.2')
    assert policy_area.label == 'd-2-12'
    assert policy_area.label_path == ['d-1-1', 'd-2-12']
    assert policy_area.html(DummyRequest()) == '<span>d-1-1 &gt; d-2-12</span>'

    policy_area = PolicyArea('1.12.121')
    assert repr(policy_area) == '1.12.121'
    assert policy_area.level == 3
    assert policy_area.descriptor == 121
    assert policy_area.descriptor_path == [1, 12, 121]
    assert policy_area.descriptor_decimal == Decimal('1.21')
    assert policy_area.label == 'd-3-121'
    assert policy_area.label_path == ['d-1-1', 'd-2-12', 'd-3-121']
    assert policy_area.html(DummyRequest()) == (
        '<span>d-1-1 &gt; d-2-12 &gt; d-3-121</span>'
    )

    assert repr(PolicyArea([1])) == '1'
    assert repr(PolicyArea([1, 12])) == '1.12'
    assert repr(PolicyArea([1, 12, 121])) == '1.12.121'

    assert repr(PolicyArea(Decimal('1'), 1)) == '1'
    assert repr(PolicyArea(Decimal('1.2'), 2)) == '1.12'
    assert repr(PolicyArea(Decimal('1.21'), 3)) == '1.12.121'

    assert PolicyArea(Decimal('10.23'), 3).label_path == [
        'd-1-10', 'd-2-102', 'd-3-1023'
    ]

    assert PolicyArea('1.12.121') == PolicyArea([1, 12, 121])


def test_principal(session):
    principal = Principal()
    assert principal


def test_vote(session):
    session.add(
        SwissVote(
            bfs_number=Decimal('100.1'),
            date=date(1990, 6, 2),
            decade=NumericRange(1990, 1999),
            legislation_number=4,
            legislation_decade=NumericRange(1990, 1994),
            title="Vote",
            keyword="Keyowrd",
            votes_on_same_day=2,
            _legal_form=1,
            initiator="Initiator",
            anneepolitique="anneepolitique",
            descriptor_1_level_1=Decimal('4'),
            descriptor_1_level_2=Decimal('4.2'),
            descriptor_1_level_3=Decimal('4.21'),
            descriptor_2_level_1=Decimal('10'),
            descriptor_2_level_2=Decimal('10.3'),
            descriptor_2_level_3=Decimal('10.35'),
            descriptor_3_level_1=Decimal('10'),
            descriptor_3_level_2=Decimal('10.3'),
            descriptor_3_level_3=Decimal('10.33'),
            _result=1,
            result_eligible_voters=2,
            result_votes_empty=3,
            result_votes_invalid=4,
            result_votes_valid=5,
            result_votes_total=6,
            result_turnout=Decimal('20.01'),
            _result_people_accepted=1,
            result_people_yeas=8,
            result_people_nays=9,
            result_people_yeas_p=Decimal('40.01'),
            _result_cantons_accepted=1,
            result_cantons_yeas=Decimal('1.5'),
            result_cantons_nays=Decimal('24.5'),
            result_cantons_yeas_p=Decimal('60.01'),
            _department_in_charge=1,
            procedure_number=Decimal('24.557'),
            _position_federal_council=1,
            _position_parliament=1,
            _position_national_council=1,
            position_national_council_yeas=10,
            position_national_council_nays=20,
            _position_council_of_states=1,
            position_council_of_states_yeas=30,
            position_council_of_states_nays=40,
            duration_federal_assembly=30,
            duration_post_federal_assembly=31,
            duration_initative_collection=32,
            duration_initative_federal_council=33,
            duration_initative_total=34,
            duration_referendum_collection=35,
            duration_referendum_total=36,
            signatures_valid=40,
            signatures_invalid=41,
            _recommendation_fdp=1,
            _recommendation_cvp=1,
            _recommendation_sps=1,
            _recommendation_svp=1,
            _recommendation_lps=2,
            _recommendation_ldu=2,
            _recommendation_evp=2,
            _recommendation_csp=3,
            _recommendation_pda=3,
            _recommendation_poch=3,
            _recommendation_gps=4,
            _recommendation_sd=4,
            _recommendation_rep=4,
            _recommendation_edu=5,
            _recommendation_fps=5,
            _recommendation_lega=5,
            _recommendation_kvp=66,
            _recommendation_glp=66,
            _recommendation_bdp=None,
            _recommendation_mcg=9999,
            _recommendation_sav=1,
            _recommendation_eco=2,
            _recommendation_sgv=3,
            _recommendation_sbv_usp=3,
            _recommendation_sgb=3,
            _recommendation_travs=3,
            _recommendation_vsa=9999,
            national_council_election_year=1990,
            national_council_share_fdp=Decimal('01.10'),
            national_council_share_cvp=Decimal('02.10'),
            national_council_share_sp=Decimal('03.10'),
            national_council_share_svp=Decimal('04.10'),
            national_council_share_lps=Decimal('05.10'),
            national_council_share_ldu=Decimal('06.10'),
            national_council_share_evp=Decimal('07.10'),
            national_council_share_csp=Decimal('08.10'),
            national_council_share_pda=Decimal('09.10'),
            national_council_share_poch=Decimal('10.10'),
            national_council_share_gps=Decimal('11.10'),
            national_council_share_sd=Decimal('12.10'),
            national_council_share_rep=Decimal('13.10'),
            national_council_share_edu=Decimal('14.10'),
            national_council_share_fps=Decimal('15.10'),
            national_council_share_lega=Decimal('16.10'),
            national_council_share_kvp=Decimal('17.10'),
            national_council_share_glp=Decimal('18.10'),
            national_council_share_bdp=Decimal('19.10'),
            national_council_share_mcg=Decimal('20.20'),
            national_council_share_ubrige=Decimal('21.20'),
            national_council_share_yeas=Decimal('22.20'),
            national_council_share_nays=Decimal('23.20'),
            national_council_share_neutral=Decimal('24.20'),
            national_council_share_vague=Decimal('25.10'),
        )
    )
    session.flush()

    vote = session.query(SwissVote).one()
    assert vote.id == 1
    assert vote.bfs_number == Decimal('100.10')
    assert vote.date == date(1990, 6, 2)
    assert vote.decade == NumericRange(1990, 1999)
    assert vote.legislation_number == 4
    assert vote.legislation_decade == NumericRange(1990, 1994)
    assert vote.title == "Vote"
    assert vote.keyword == "Keyowrd"
    assert vote.votes_on_same_day == 2
    assert vote._legal_form == 1
    assert vote.legal_form == "Mandatory referendum"
    assert vote.initiator == "Initiator"
    assert vote.anneepolitique == "anneepolitique"
    assert vote.descriptor_1_level_1 == Decimal('4')
    assert vote.descriptor_1_level_2 == Decimal('4.2')
    assert vote.descriptor_1_level_3 == Decimal('4.21')
    assert vote.descriptor_2_level_1 == Decimal('10')
    assert vote.descriptor_2_level_2 == Decimal('10.3')
    assert vote.descriptor_2_level_3 == Decimal('10.35')
    assert vote.descriptor_3_level_1 == Decimal('10')
    assert vote.descriptor_3_level_2 == Decimal('10.3')
    assert vote.descriptor_3_level_3 == Decimal('10.33')
    assert vote._result == 1
    assert vote.result == "Accepted"
    assert vote.result_eligible_voters == 2
    assert vote.result_votes_empty == 3
    assert vote.result_votes_invalid == 4
    assert vote.result_votes_valid == 5
    assert vote.result_votes_total == 6
    assert vote.result_turnout == Decimal('20.01')
    assert vote._result_people_accepted == 1
    assert vote.result_people_accepted == "Accepted"
    assert vote.result_people_yeas == 8
    assert vote.result_people_nays == 9
    assert vote.result_people_yeas_p == Decimal('40.01')
    assert vote._result_cantons_accepted == 1
    assert vote.result_cantons_accepted == "Accepted"
    assert vote.result_cantons_yeas == Decimal('1.5')
    assert vote.result_cantons_nays == Decimal('24.5')
    assert vote.result_cantons_yeas_p == Decimal('60.01')
    assert vote._department_in_charge == 1
    assert vote.department_in_charge == \
        "Federal Department of Foreign Affairs (FDFA)"
    assert vote.procedure_number == Decimal('24.557')
    assert vote._position_federal_council == 1
    assert vote.position_federal_council == "Accepting"
    assert vote._position_parliament == 1
    assert vote.position_parliament == "Accepting"
    assert vote._position_national_council == 1
    assert vote.position_national_council == "Accepting"
    assert vote.position_national_council_yeas == 10
    assert vote.position_national_council_nays == 20
    assert vote._position_council_of_states == 1
    assert vote.position_council_of_states == "Accepting"
    assert vote.position_council_of_states_yeas == 30
    assert vote.position_council_of_states_nays == 40
    assert vote.duration_federal_assembly == 30
    assert vote.duration_post_federal_assembly == 31
    assert vote.duration_initative_collection == 32
    assert vote.duration_initative_federal_council == 33
    assert vote.duration_initative_total == 34
    assert vote.duration_referendum_collection == 35
    assert vote.duration_referendum_total == 36
    assert vote.signatures_valid == 40
    assert vote.signatures_invalid == 41
    assert vote._recommendation_fdp == 1
    assert vote._recommendation_cvp == 1
    assert vote._recommendation_sps == 1
    assert vote._recommendation_svp == 1
    assert vote._recommendation_lps == 2
    assert vote._recommendation_ldu == 2
    assert vote._recommendation_evp == 2
    assert vote._recommendation_csp == 3
    assert vote._recommendation_pda == 3
    assert vote._recommendation_poch == 3
    assert vote._recommendation_gps == 4
    assert vote._recommendation_sd == 4
    assert vote._recommendation_rep == 4
    assert vote._recommendation_edu == 5
    assert vote._recommendation_fps == 5
    assert vote._recommendation_lega == 5
    assert vote._recommendation_kvp == 66
    assert vote._recommendation_glp == 66
    assert vote._recommendation_bdp is None
    assert vote._recommendation_mcg == 9999
    assert vote._recommendation_sav == 1
    assert vote._recommendation_eco == 2
    assert vote._recommendation_sgv == 3
    assert vote._recommendation_sbv_usp == 3
    assert vote._recommendation_sgb == 3
    assert vote._recommendation_travs == 3
    assert vote._recommendation_vsa == 9999
    assert vote.national_council_election_year == 1990
    assert vote.national_council_share_fdp == Decimal('01.10')
    assert vote.national_council_share_cvp == Decimal('02.10')
    assert vote.national_council_share_sp == Decimal('03.10')
    assert vote.national_council_share_svp == Decimal('04.10')
    assert vote.national_council_share_lps == Decimal('05.10')
    assert vote.national_council_share_ldu == Decimal('06.10')
    assert vote.national_council_share_evp == Decimal('07.10')
    assert vote.national_council_share_csp == Decimal('08.10')
    assert vote.national_council_share_pda == Decimal('09.10')
    assert vote.national_council_share_poch == Decimal('10.10')
    assert vote.national_council_share_gps == Decimal('11.10')
    assert vote.national_council_share_sd == Decimal('12.10')
    assert vote.national_council_share_rep == Decimal('13.10')
    assert vote.national_council_share_edu == Decimal('14.10')
    assert vote.national_council_share_fps == Decimal('15.10')
    assert vote.national_council_share_lega == Decimal('16.10')
    assert vote.national_council_share_kvp == Decimal('17.10')
    assert vote.national_council_share_glp == Decimal('18.10')
    assert vote.national_council_share_bdp == Decimal('19.10')
    assert vote.national_council_share_mcg == Decimal('20.20')
    assert vote.national_council_share_ubrige == Decimal('21.20')
    assert vote.national_council_share_yeas == Decimal('22.20')
    assert vote.national_council_share_nays == Decimal('23.20')
    assert vote.national_council_share_neutral == Decimal('24.20')
    assert vote.national_council_share_vague == Decimal('25.10')

    assert vote.policy_areas == [
        PolicyArea('4.42.421'),
        PolicyArea('10.103.1035'),
        PolicyArea('10.103.1033')
    ]

    assert list(vote.recommendations_parties.keys()) == [
        'Yea', 'Nay', 'None', 'Empty', 'Free vote', 'Neutral'
    ]
    assert vote.recommendations_parties['Yea'] == [
        Actor('cvp'),
        Actor('fdp'),
        Actor('sps'),
        Actor('svp'),
    ]
    assert vote.recommendations_parties['Nay'] == [
        Actor('evp'),
        Actor('ldu'),
        Actor('lps'),
    ]
    assert vote.recommendations_parties['None'] == [
        Actor('csp'),
        Actor('pda'),
        Actor('poch'),
    ]
    assert vote.recommendations_parties['Empty'] == [
        Actor('gps'),
        Actor('rep'),
        Actor('sd'),
    ]
    assert vote.recommendations_parties['Free vote'] == [
        Actor('edu'),
        Actor('fps'),
        Actor('lega'),
    ]
    assert vote.recommendations_parties['Neutral'] == [
        Actor('glp'),
        Actor('kvp')
    ]

    assert list(vote.recommendations_associations.keys()) == [
        'Yea', 'Nay', 'None'
    ]
    assert vote.recommendations_associations['Yea'] == [Actor('sav')]
    assert vote.recommendations_associations['Nay'] == [Actor('eco')]
    assert vote.recommendations_associations['None'] == [
        Actor('sbv-usp'),
        Actor('sgb'),
        Actor('sgv'),
        Actor('travs'),
    ]

    assert vote.has_national_council_share_data is True


def test_vote_codes():
    assert SwissVote.codes('legal_form')[2] == "Optional referendum"
    assert SwissVote.codes('result')[0] == "Rejected"
    assert SwissVote.codes('result_people_accepted')[0] == "Rejected"
    assert SwissVote.codes('result_cantons_accepted')[0] == "Rejected"
    assert SwissVote.codes('department_in_charge')[8] == \
        "Federal Chancellery (FCh)"
    assert SwissVote.codes('position_federal_council')[3] == "Neutral"
    assert SwissVote.codes('position_parliament')[2] == "Rejecting"
    assert SwissVote.codes('position_national_council')[2] == "Rejecting"
    assert SwissVote.codes('position_council_of_states')[2] == "Rejecting"
    assert SwissVote.codes('recommendation')[5] == "Free vote"


def test_vote_attachments(swissvotes_app, attachments):
    session = swissvotes_app.session()
    session.add(
        SwissVote(
            bfs_number=Decimal('100.1'),
            date=date(1990, 6, 2),
            decade=NumericRange(1990, 1999),
            legislation_number=4,
            legislation_decade=NumericRange(1990, 1994),
            title="Vote",
            keyword="Keyowrd",
            votes_on_same_day=2,
            _legal_form=1,
        )
    )
    session.flush()

    vote = session.query(SwissVote).one()
    assert vote.voting_text is None
    assert vote.federal_council_message is None
    assert vote.parliamentary_debate is None
    assert vote.voting_booklet is None
    assert vote.resolution is None
    assert vote.realization is None
    assert vote.files == []
    assert vote.searchable_text_de_CH is None
    assert vote.searchable_text_fr_CH is None

    assert vote.indexed_files == {
        'federal_council_message',
        'parliamentary_debate',
        'realization',
        'voting_text',
    }

    vote.voting_text = attachments['voting_text']
    session.flush()

    assert len(vote.files) == 1
    assert vote.voting_text.name == 'voting_text-de_CH'
    assert "abstimmungstex" in vote.searchable_text_de_CH
    assert vote.searchable_text_fr_CH is None

    swissvotes_app.session_manager.current_locale = 'fr_CH'

    vote.realization = attachments['realization']
    session.flush()

    assert len(vote.files) == 2
    assert vote.voting_text is None
    assert vote.realization.name == 'realization-fr_CH'
    assert "réalis" in vote.searchable_text_fr_CH



from webtest import TestApp as Client


def test_view_home(swissvotes_app):
    client = Client(swissvotes_app)
    home = client.get('/').follow()
    assert "Startseite" in home
    assert home.request.url.endswith('page/home')



from cgi import FieldStorage
from io import BytesIO
from onegov.core.crypto import random_token
from onegov.file.utils import as_fileintent
from onegov.form import Form
from onegov.form.fields import UploadField
from onegov.form.validators import FileSizeLimit
from onegov.form.validators import WhitelistedMimeType
from onegov.swissvotes import _
from onegov.swissvotes.models import SwissVoteFile


class AttachmentsForm(Form):

    voting_text = UploadField(
        label=_("Voting text"),
        validators=[
            WhitelistedMimeType({'application/pdf'}),
            FileSizeLimit(25 * 1024 * 1024)
        ]
    )

    federal_council_message = UploadField(
        label=_("Federal council message"),
        validators=[
            WhitelistedMimeType({'application/pdf'}),
            FileSizeLimit(25 * 1024 * 1024)
        ]
    )

    parliamentary_debate = UploadField(
        label=_("Parliamentary debate"),
        validators=[
            WhitelistedMimeType({'application/pdf'}),
            FileSizeLimit(25 * 1024 * 1024)
        ]
    )

    voting_booklet = UploadField(
        label=_("Voting booklet"),
        validators=[
            WhitelistedMimeType({'application/pdf'}),
            FileSizeLimit(25 * 1024 * 1024)
        ]
    )

    resolution = UploadField(
        label=_("Resolution"),
        validators=[
            WhitelistedMimeType({'application/pdf'}),
            FileSizeLimit(25 * 1024 * 1024)
        ]
    )

    realization = UploadField(
        label=_("Realization"),
        validators=[
            WhitelistedMimeType({'application/pdf'}),
            FileSizeLimit(25 * 1024 * 1024)
        ]
    )

    def update_model(self, model):
        locale = self.request.locale
        if self.voting_text.action == 'delete':
            del model.voting_text
        if self.voting_text.action == 'replace':
            if self.voting_text.data:
                voting_text = SwissVoteFile(id=random_token())
                voting_text.reference = as_fileintent(
                    self.voting_text.raw_data[-1].file,
                    f'voting_text-{locale}'
                )
                model.voting_text = voting_text

        if self.federal_council_message.action == 'delete':
            del model.federal_council_message
        if self.federal_council_message.action == 'replace':
            if self.federal_council_message.data:
                federal_council_message = SwissVoteFile(id=random_token())
                federal_council_message.reference = as_fileintent(
                    self.federal_council_message.raw_data[-1].file,
                    f'federal_council_message-{locale}'
                )
                model.federal_council_message = federal_council_message

        if self.parliamentary_debate.action == 'delete':
            del model.parliamentary_debate
        if self.parliamentary_debate.action == 'replace':
            if self.parliamentary_debate.data:
                parliamentary_debate = SwissVoteFile(id=random_token())
                parliamentary_debate.reference = as_fileintent(
                    self.parliamentary_debate.raw_data[-1].file,
                    f'parliamentary_debate-{locale}'
                )
                model.parliamentary_debate = parliamentary_debate

        if self.voting_booklet.action == 'delete':
            del model.voting_booklet
        if self.voting_booklet.action == 'replace':
            if self.voting_booklet.data:
                voting_booklet = SwissVoteFile(id=random_token())
                voting_booklet.reference = as_fileintent(
                    self.voting_booklet.raw_data[-1].file,
                    f'voting_booklet-{locale}'
                )
                model.voting_booklet = voting_booklet

        if self.resolution.action == 'delete':
            del model.resolution
        if self.resolution.action == 'replace':
            if self.resolution.data:
                resolution = SwissVoteFile(id=random_token())
                resolution.reference = as_fileintent(
                    self.resolution.raw_data[-1].file,
                    f'resolution-{locale}'
                )
                model.resolution = resolution

        if self.realization.action == 'delete':
            del model.realization
        if self.realization.action == 'replace':
            if self.realization.data:
                realization = SwissVoteFile(id=random_token())
                realization.reference = as_fileintent(
                    self.realization.raw_data[-1].file,
                    f'realization-{locale}'
                )
                model.realization = realization

    def apply_model(self, model):
        def assign_file(field, file):
            # todo: implement something better
            fs = FieldStorage()
            fs.file = BytesIO(file.reference.file.read())
            fs.type = file.reference.content_type
            fs.filename = file.reference.filename
            field.data = field.process_fieldstorage(fs)

        if self.model.voting_text:
            assign_file(
                self.voting_text,
                self.model.voting_text
            )
        if self.model.federal_council_message:
            assign_file(
                self.federal_council_message,
                self.model.federal_council_message
            )
        if self.model.parliamentary_debate:
            assign_file(
                self.parliamentary_debate,
                self.model.parliamentary_debate
            )
        if self.model.voting_booklet:
            assign_file(
                self.voting_booklet,
                self.model.voting_booklet
            )
        if self.model.resolution:
            assign_file(
                self.resolution,
                self.model.resolution
            )
        if self.model.realization:
            assign_file(
                self.realization,
                self.model.realization
            )
